"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const Promise = require('bluebird');

const {
  ActionTransport
} = require('@microfleet/core');

const challenge = require('../utils/challenges/challenge');

const {
  getInternalData
} = require('../utils/userData');

const isActive = require('../utils/is-active');

const isBanned = require('../utils/is-banned');

const hasNotPassword = require('../utils/has-no-password');

const {
  USERS_ACTION_DISPOSABLE_PASSWORD,
  USERS_USERNAME_FIELD
} = require('../constants');
/**
 * @api {amqp} <prefix>.disposable-password Request disposable password
 * @apiVersion 1.0.0
 * @apiName DisposablePassword
 * @apiGroup Users
 *
 * @apiDescription This method allowes to get disposable password.
 *
 * @apiSchema {jsonschema=../../schemas/disposable-password.json} apiParam
 */


module.exports = function disposablePassword(request) {
  const {
    challengeType,
    id
  } = request.params;
  const {
    [challengeType]: tokenOptions
  } = this.config.token;
  return Promise.bind(this, id).then(getInternalData).tap(isActive).tap(isBanned).tap(hasNotPassword).then(data => [challengeType, _objectSpread({
    id: data[USERS_USERNAME_FIELD],
    action: USERS_ACTION_DISPOSABLE_PASSWORD
  }, tokenOptions)]).spread(challenge).then(response => {
    const {
      uid
    } = response.context.token;
    return {
      requested: true,
      uid
    };
  });
};

module.exports.transports = [ActionTransport.amqp, ActionTransport.internal];