"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const Promise = require('bluebird');

const generateEmail = require('../utils/challenges/email/generate.js');

const {
  INVITATIONS_INDEX,
  TOKEN_METADATA_FIELD_METADATA,
  TOKEN_METADATA_FIELD_CONTEXT,
  TOKEN_METADATA_FIELD_SENDED_AT,
  USERS_ACTION_INVITE
} = require('../constants.js');
/**
 * @api {amqp} <prefix>.invite Generate User Invitation
 * @apiVersion 1.0.0
 * @apiName inviteuser
 * @apiGroup Users
 *
 * @apiDescription Send an email with special registration link, which embeds a token. When supplied during registration
 * process it will allow user to not verify him or herself, as well as add extra metadata to their profile when registration
 * is complete. Can only be used once and could have an expiration date.
 *
 * @apiParam (Payload) {String} email - used to send the invitation
 * @apiParam (Payload) {Object} [ctx] - context to be passed into email
 * @apiParam (Payload) {Number} [throttle] - if set, rejects to send another invite to the same email during that time
 * @apiParam (Payload) {Number} [ttl] - if set, invitation will expire in that time
 * @apiParam (Payload) {Object} [nodemailer] - if set, would be passed to nodemailer options
 * @apiParam (Payload) {Object} [metadata] - metadata to be added to the user upon registration
 * @apiParam (Payload) {Object} [metadata.*] - `*` is a namespace for which metadata would be added
 * @apiParam (Payload) {Mixed}  [metadata.*.*] - `*` is a key, for which associated value must be passed. It can be anything that
 * could be serialized using JSON.stringify()
 *
 */


module.exports = function generateInvite(request) {
  const {
    redis,
    tokenManager
  } = this;
  const {
    email,
    ctx = {},
    throttle = 0,
    ttl = 0,
    metadata = {},
    nodemailer = {}
  } = request.params;
  const now = Date.now(); // do not throttle

  return tokenManager.create({
    id: email,
    action: USERS_ACTION_INVITE,
    regenerate: true,
    ttl,
    // defaults to never expiring
    throttle,
    // defaults to no throttle
    metadata: {
      [TOKEN_METADATA_FIELD_METADATA]: metadata,
      [TOKEN_METADATA_FIELD_CONTEXT]: ctx,
      [TOKEN_METADATA_FIELD_SENDED_AT]: now
    }
  }).then(token => Promise.bind(this, [email, USERS_ACTION_INVITE, _objectSpread({}, ctx, {
    token
  }), {
    send: true
  }, nodemailer]).spread(generateEmail).tap(() => redis.sadd(INVITATIONS_INDEX, email)));
};

module.exports.transports = [require('@microfleet/core').ActionTransport.amqp];