"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const Promise = require('bluebird');

const {
  getInternalData
} = require('../utils/userData');

const isActive = require('../utils/is-active');

const isBanned = require('../utils/is-banned');

const hasPassword = require('../utils/has-password');

const getMetadata = require('../utils/get-metadata');

const challenge = require('../utils/challenges/challenge');

const {
  USERS_ACTION_PASSWORD,
  USERS_ACTION_RESET,
  USERS_ID_FIELD,
  USERS_USERNAME_FIELD
} = require('../constants.js');
/**
 * @api {amqp} <prefix>.requestPassword Reset Password
 * @apiVersion 1.0.0
 * @apiName PasswordReset
 * @apiGroup Users
 *
 * @apiDescription Allows one either to request new password instantly, or generate a challenge.
 * In the first case would send new password to email instantly and will change it in the system.
 * Use-case is discouraged, because it can be used to DOS account (throttling not implemented).
 * Second case sends reset token to email and it can be used in `updatePassword` endpoint
 * alongside new password to generate it
 *
 * @apiSchema {jsonschema=../../schemas/requestPassword.json} apiParam
 */


module.exports = function requestPassword(request) {
  const {
    challengeType,
    username: usernameOrAlias,
    generateNewPassword
  } = request.params;
  const {
    [challengeType]: tokenOptions
  } = this.config.token;
  const {
    defaultAudience
  } = this.config.jwt;
  const action = generateNewPassword ? USERS_ACTION_PASSWORD : USERS_ACTION_RESET; // TODO: make use of remoteip in security logs?
  // var remoteip = request.params.remoteip;

  return Promise.bind(this, usernameOrAlias).then(getInternalData).tap(isActive).tap(isBanned).tap(hasPassword).then(data => [data[USERS_ID_FIELD], defaultAudience]).spread(getMetadata).get(defaultAudience).then(meta => [challengeType, _objectSpread({
    id: meta[USERS_USERNAME_FIELD],
    action
  }, tokenOptions), meta]).spread(challenge).return({
    success: true
  });
};

module.exports.transports = [require('@microfleet/core').ActionTransport.amqp];