"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const is = require('is');

const assert = require('assert');

const defaults = require('lodash/defaults');

const strategies = require('./providers'); // helpers


const {
  isArray
} = Array;

const stringToArray = (scope, scopeSeparator) => isArray(scope) ? scope : scope.split(scopeSeparator);

const hapiOauthHandler = (request, h) => {
  const {
    redirectUri
  } = request.response; // redirect if redirectURI is present

  if (redirectUri) {
    return h.redirect(redirectUri);
  }

  return h.continue;
};
/**
 * This function initiates bell plugin strategies on hapi.js webserver
 * @param {Hapi} server
 * @param {Objecr} config
 */


module.exports = function OauthHandler(server, config) {
  assert.ok(config.oauth, 'oauth configuration must be present');
  assert.ok(config.oauth.providers, 'oauth configuration must include providers');
  server.ext('onPreResponse', hapiOauthHandler);

  for (const [name, options] of Object.entries(config.oauth.providers)) {
    const strategy = strategies[name];

    if (options.enabled === false) {
      continue; // eslint-disable-line no-continue
    }

    if (!strategy) {
      throw new Error(`OAuth: unknown strategy ${name}`);
    }

    let provider;
    const defaultOptions = strategy.options;

    const {
      scope,
      fields,
      profileHandler,
      scopeSeparator,
      apiVersion,
      enabled,
      retryOnMissingPermissions,
      isSameSite
    } = options,
          rest = _objectWithoutProperties(options, ["scope", "fields", "profileHandler", "scopeSeparator", "apiVersion", "enabled", "retryOnMissingPermissions", "isSameSite"]); // make sure runtime params are allowed in we want to retry as we need to defined dynamic
    // redirect params


    if (retryOnMissingPermissions === true) {
      rest.allowRuntimeProviderParams = true;
    }

    if (defaultOptions) {
      const configuredOptions = {
        name,
        scope: stringToArray(scope),
        scopeSeparator
      };

      if (is.fn(defaultOptions)) {
        provider = defaultOptions(_objectSpread({}, configuredOptions, {
          apiVersion,
          fields,
          profileHandler
        }));
      } else {
        provider = defaults(configuredOptions, defaultOptions);
      }
    } else {
      // use bell defaults
      provider = name;
    } // init strategy


    server.auth.strategy(name, 'bell', _objectSpread({
      provider
    }, rest));

    if (isSameSite || isSameSite === false) {
      // NOTE: this overwrites the default from bell
      // must be specified, defaults to mf_bfb
      server.states.cookies[rest.cookie].isSameSite = rest.isSameSite;
    }
  }

  return server;
};