"use strict";

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const Errors = require('common-errors');

const Boom = require('@hapi/boom');

exports.Redirect = Errors.helpers.generateClass('Redirect', {
  args: ['redirectUri'],

  generateMessage() {
    return `${this.name}: ${this.redirectUri}`;
  }

});
const OAuthError = exports.OAuthError = Errors.helpers.generateClass('OAuthError', {
  args: ['message', 'inner_error']
});
/**
 * Removes `@hapi/boom` attributes
 * @param error
 * @returns {*|{stack: *, data: *, name: *, message: *}}
 */

function stripBoomAttrs(error) {
  const {
    message,
    name,
    stack,
    data: errorData
  } = error;
  let data;
  /* assuming that 'data' not passed or it's not and object when Boom.error created. */

  if (errorData !== null && typeof errorData === 'object') {
    /* Remove http.IncomingMessage - @hapi/wreck adds it when error is coming from response. */
    if (errorData.isResponseError) {
      const {
        res
      } = errorData,
            otherData = _objectWithoutProperties(errorData, ["res"]);

      data = otherData;
    } else {
      ({
        data
      } = error);
    }
  } else {
    data = errorData;
  }
  /* Recursive check */


  if (Boom.isBoom(error.data)) {
    data = stripBoomAttrs(error.data);
  }

  return {
    message,
    name,
    stack,
    data
  };
}
/**
 * OAuthError.toJSON custom serialization
 * Returns simplified object
 * @returns {*}
 */


OAuthError.prototype.toJSON = function toJSON() {
  const {
    inner_error: innerError,
    message,
    name,
    stack
  } = this;
  let innerErrorJsonObj;
  /* Boom error contains additional data */

  if (Boom.isBoom(innerError)) {
    innerErrorJsonObj = stripBoomAttrs(innerError);
  } else {
    innerErrorJsonObj = innerError;
  }

  return {
    message,
    name,
    stack,
    inner_error: innerErrorJsonObj
  };
};