"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const path = require('path');

const {
  routerExtension,
  ActionTransport
} = require('@microfleet/core');
/**
 * Loads existing auth strategies
 */


const strategies = require('../auth/strategies');
/**
 * This extension defaults schemas to the name of the action
 * @type {Function}
 */


const autoSchema = routerExtension('validate/schemaLessAction');
/**
 * Provides audit log for every performed action
 * @type {Function}
 */

const auditLog = routerExtension('audit/log');
/**
 * Provides prometheus metrics
 * @type {Function}
 */

const metrics = routerExtension('audit/metrics');
/**
 * Catches errors from oauth.facebook and wraps them into HTML
 * @type {Function}
 */

const preResponse = require('../auth/oauth/pre-response');
/**
 * Specifies configuration for the router of the microservice
 * @type {Object}
 */


exports.router = {
  routes: {
    directory: path.resolve(__dirname, '../actions'),
    prefix: 'users',
    transports: [ActionTransport.amqp, ActionTransport.http, ActionTransport.internal],
    enabledGenericActions: ['health']
  },
  extensions: {
    enabled: ['preRequest', 'postRequest', 'postValidate', 'preResponse', 'postResponse'],
    register: [autoSchema, preResponse, auditLog(), metrics()]
  },
  auth: {
    strategies: _objectSpread({}, strategies)
  }
};