"use strict";

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint-disable no-mixed-operators */
const Promise = require('bluebird');

const mapValues = require('lodash/mapValues');

const redisKey = require('../key.js');

const getUserId = require('../userData/get-user-id');

const sendInviteMail = require('./send-invite-email');

const getInternalData = require('./get-internal-data');

const registerOrganizationMembers = require('./register-organization-members');

const handlePipeline = require('../pipeline-error');

const {
  ORGANIZATIONS_MEMBERS,
  USERS_METADATA,
  ORGANIZATIONS_NAME_FIELD,
  USERS_ACTION_ORGANIZATION_INVITE,
  USERS_ACTION_ORGANIZATION_REGISTER,
  ORGANIZATIONS_ID_FIELD
} = require('../../constants.js');

const JSONStringify = data => JSON.stringify(data);
/**
 * Updates metadata on a organization object
 * @param  {Object} opts
 * @return {Promise}
 */


async function addOrganizationMembers(opts) {
  const {
    redis
  } = this;
  const {
    organizationId,
    members,
    audience
  } = opts;
  const registeredMembers = [];
  const notRegisteredMembers = [];
  const filterMembersJob = members.map(async member => {
    try {
      const userId = await getUserId.call(this, member.email);
      registeredMembers.push(_objectSpread({}, member, {
        id: userId
      }));
    } catch (e) {
      notRegisteredMembers.push(member);
    }
  });
  await Promise.all(filterMembersJob);
  const createdMembers = await registerOrganizationMembers.call(this, notRegisteredMembers);
  const pipe = redis.pipeline();
  const membersKey = redisKey(organizationId, ORGANIZATIONS_MEMBERS);
  const organizationMembers = registeredMembers.concat(createdMembers);
  organizationMembers.forEach((_ref) => {
    let {
      password
    } = _ref,
        member = _objectWithoutProperties(_ref, ["password"]);

    const memberKey = redisKey(organizationId, ORGANIZATIONS_MEMBERS, member.id);
    const memberOrganizations = redisKey(member.id, USERS_METADATA, audience);
    member.username = member.email;
    member.invited = Date.now();
    member.accepted = password ? Date.now() : null;
    member.permissions = member.permissions || [];
    const stringifyMember = mapValues(member, JSONStringify);
    pipe.hmset(memberKey, stringifyMember);
    pipe.hset(memberOrganizations, organizationId, stringifyMember.permissions);
    pipe.zadd(membersKey, stringifyMember.invited, memberKey);
  });
  await pipe.exec().then(handlePipeline);
  const organization = await getInternalData.call(this, organizationId);
  const membersIdsJob = [];

  for (const member of organizationMembers) {
    membersIdsJob.push(sendInviteMail.call(this, {
      email: member.email,
      action: member.password ? USERS_ACTION_ORGANIZATION_REGISTER : USERS_ACTION_ORGANIZATION_INVITE,
      ctx: {
        firstName: member.firstName,
        lastName: member.lastName,
        password: member.password,
        email: member.email,
        organizationId: organization[ORGANIZATIONS_ID_FIELD],
        organization: organization[ORGANIZATIONS_NAME_FIELD]
      }
    }));
  }

  return Promise.all(membersIdsJob);
}

module.exports = addOrganizationMembers;